<?php
/**
 * Smarty plugin
 * @package Smarty
 * @subpackage plugins
 */


/**
 * Smarty truncate modifier plugin
 *
 * Type:     modifier<br>
 * Name:     truncate<br>
 * Purpose:  Truncate a string to a certain length if necessary,
 *           optionally splitting in the middle of a word, and
 *           appending the $etc string or inserting $etc into the middle.
 * @link http://smarty.php.net/manual/en/language.modifier.truncate.php
 *          truncate (Smarty online manual)
 * @author   Monte Ohrt <monte at ohrt dot com>
 * @param string
 * @param integer
 * @param string
 * @param boolean
 * @param boolean
 * @return string
 */
function smarty_modifier_truncate($string, $length = 80, $etc = '...',
                                  $break_words = false, $middle = false)
{
    if ($length == 0)
        return '';

    if (mb_strlen($string, 'UTF-8') > $length) {
        $string_orig = $string;
        
        $length -= min($length, mb_strlen($etc, 'UTF-8'));
        if (!$break_words && !$middle) {
            $string = mb_ereg_replace('/\s+?(\S+)?$/', '', mb_substr($string, 0, $length+1, 'UTF-8'));
        }

        if (!$middle) {
            $string = mb_substr($string, 0, $length, 'UTF-8');

            if (preg_match('!&([a-z]+)$!is', $string, $matches)) {
                $l = mb_strlen($string, 'UTF-8') - mb_strlen($matches[1], 'UTF-8') - 1;
                $tmp = mb_substr($string_orig, $l, 655350, 'UTF-8');

                if (preg_match('!^&[a-z]+;!is', $tmp)) {
                    $string = mb_substr($string, 0, $l, 'UTF-8');
                }
            } elseif (preg_match('!<([a-z]+[^>]*)$!is', $string, $matches)) {
                $l = mb_strlen($string, 'UTF-8') - mb_strlen($matches[1], 'UTF-8') - 1;
                $string = mb_substr($string, 0, $l);
            } else {
                $string = rtrim($string, '<');
            }

            $string .= $etc;
        } else {
            $string = mb_substr($string, 0, $length/2, 'UTF-8') . $etc . mb_substr($string, -$length/2, 'UTF-8');
        }
    }
    
    return $string;
}

/* vim: set expandtab: */

?>
